@@ -1,4 +1,4 @@
-from typing import Optional, List
+from typing import Optional, List, Set
 
 from robocorp_ls_core.lsp import CompletionItemTypedDict
 from robocorp_ls_core.protocols import check_implements, IDocumentSelection
@@ -17,10 +17,7 @@ from robotframework_ls.impl.protocols import (
 from robotframework_ls.impl.text_utilities import normalize_robot_name
 from robotframework_ls.impl.variable_types import (
     VariableFoundFromToken,
-    VariableFoundFromPythonAst,
     VariableFoundFromYaml,
-    VariableFoundFromSettings,
-    VariableFoundFromBuiltins,
 )
 
 
@@ -93,45 +90,15 @@ class _Collector(AbstractVariablesCollector):
         _: IVariablesCollector = check_implements(self)
 
 
-def _collect_completions_from_ast(
-    ast, completion_context: ICompletionContext, collector: IVariablesCollector
+def _collect_variables_from_set_keywords(
+    ast,
+    completion_context: ICompletionContext,
+    collector: IVariablesCollector,
+    accept_sets_in: Set[str],
 ):
-    from robotframework_ls.impl import ast_utils
-    from robotframework_ls.impl.variable_resolve import robot_search_variable
-
-    completion_context.check_cancelled()
     from robot.api import Token
+    from robotframework_ls.impl import ast_utils
 
-    for variable_node_info in completion_context.get_all_variables():
-        variable_node = variable_node_info.node
-        token = variable_node.get_token(Token.VARIABLE)
-        if token is None:
-            continue
-
-        variable_match = robot_search_variable(token.value)
-        # Filter out empty base
-        if variable_match is None or not variable_match.base:
-            continue
-
-        if collector.accepts(variable_match.base):
-            base_token = ast_utils.convert_variable_match_base_to_token(
-                token, variable_match
-            )
-            variable_found = VariableFoundFromToken(
-                completion_context,
-                base_token,
-                variable_node.value,
-                variable_name=variable_match.base,
-            )
-            collector.on_variable(variable_found)
-
-    accept_sets_in = {
-        normalize_robot_name("Set Task Variable"),
-        normalize_robot_name("Set Test Variable"),
-        normalize_robot_name("Set Suite Variable"),
-        normalize_robot_name("Set Global Variable"),
-    }
-    ast = completion_context.get_ast()
     for keyword_usage in ast_utils.iter_keyword_usage_tokens(
         ast, collect_args_as_keywords=True
     ):
@@ -147,14 +114,24 @@ def _collect_completions_from_ast(
                         break
 
             if var_name_tok is not None:
-                variable_match = robot_search_variable(var_name_tok.value)
-                # Filter out empty base
-                if variable_match is None or not variable_match.base:
-                    continue
+                var_name = var_name_tok.value
+                start_offset = var_name_tok.col_offset
 
-                if collector.accepts(variable_match.base):
-                    base_token = ast_utils.convert_variable_match_base_to_token(
-                        var_name_tok, variable_match
+                if var_name.startswith("\\"):
+                    var_name = var_name[1:]
+                    start_offset += 1
+
+                if var_name.startswith(("@", "$", "&")):
+                    var_name = var_name[1:]
+                    start_offset += 1
+
+                if var_name.startswith("{") and var_name.endswith("}"):
+                    var_name = var_name[1:-1]
+                    start_offset += 1
+
+                if collector.accepts(var_name):
+                    base_token = ast_utils.copy_token_replacing(
+                        var_name_tok, col_offset=var_name_tok.col_offset, value=var_name
                     )
                     variable_value = ""
                     if var_value_tok is not None:
@@ -164,7 +141,7 @@ def _collect_completions_from_ast(
                         completion_context,
                         base_token,
                         variable_value,
-                        variable_name=variable_match.base,
+                        variable_name=var_name,
                     )
                     collector.on_variable(variable_found)
 
@@ -176,9 +153,24 @@ def _collect_current_doc_variables(
     :param CompletionContext completion_context:
     """
     # Get keywords defined in the file itself
+    completion_context.check_cancelled()
 
-    ast = completion_context.get_ast()
-    _collect_completions_from_ast(ast, completion_context, collector)
+    for (
+        variable_found
+    ) in completion_context.get_doc_normalized_var_name_to_var_found().values():
+        if collector.accepts(variable_found.variable_name):
+            collector.on_variable(variable_found)
+
+    accept_sets_in = {
+        normalize_robot_name("Set Task Variable"),
+        normalize_robot_name("Set Test Variable"),
+        normalize_robot_name("Set Suite Variable"),
+        normalize_robot_name("Set Global Variable"),
+    }
+
+    _collect_variables_from_set_keywords(
+        completion_context.get_ast(), completion_context, collector, accept_sets_in
+    )
 
 
 def _collect_resource_imports_variables(
@@ -189,7 +181,7 @@ def _collect_resource_imports_variables(
         if resource_doc is None:
             continue
         new_ctx = completion_context.create_copy(resource_doc)
-        _collect_variables_from_document_context(new_ctx, collector)
+        _collect_global_variables_from_document_context(new_ctx, collector)
 
 
 def _collect_variables_from_variable_import_doc(
@@ -199,48 +191,13 @@ def _collect_variables_from_variable_import_doc(
         if variable_import_doc.path.lower().endswith(".py"):
             python_ast = variable_import_doc.get_python_ast()
             if python_ast is not None:
-                import ast as ast_module
+                from robotframework_ls.impl.variable_completions_from_py import (
+                    collect_variables_from_python_ast,
+                )
 
-                for node in python_ast.body:
-                    if isinstance(node, ast_module.Assign):
-                        for target in node.targets:
-                            if isinstance(target, ast_module.Name):
-                                varname = target.id
-                                if collector.accepts(varname):
-                                    value = ""
-                                    try:
-                                        # Only available for Python 3.8 onwards...
-                                        end_lineno = getattr(
-                                            node.value, "end_lineno", None
-                                        )
-                                        if end_lineno is None:
-                                            end_lineno = node.value.lineno
-
-                                        # Only available for Python 3.8 onwards...
-                                        end_col_offset = getattr(
-                                            node.value, "end_col_offset", None
-                                        )
-                                        if end_col_offset is None:
-                                            end_col_offset = 99999999
-                                        value = variable_import_doc.get_range(
-                                            node.value.lineno - 1,
-                                            node.value.col_offset,
-                                            end_lineno - 1,
-                                            end_col_offset,
-                                        )
-                                    except:
-                                        log.exception()
-
-                                    variable_found = VariableFoundFromPythonAst(
-                                        variable_import_doc.path,
-                                        target.lineno - 1,
-                                        target.col_offset,
-                                        target.lineno - 1,
-                                        target.col_offset + len(target.id),
-                                        value,
-                                        variable_name=varname,
-                                    )
-                                    collector.on_variable(variable_found)
+                collect_variables_from_python_ast(
+                    python_ast, variable_import_doc, collector
+                )
 
         elif variable_import_doc.path.lower().endswith((".yaml", ".yml")):
             dct_contents = variable_import_doc.get_yaml_contents()
@@ -284,7 +241,72 @@ def _collect_variables_from_variable_import_doc(
         log.exception()
 
 
-def _collect_variables_from_document_context(
+def collect_global_variables_from_document_dependencies(
+    completion_context: ICompletionContext,
+    collector: IVariablesCollector,
+):
+    dependency_graph = completion_context.collect_dependency_graph()
+
+    for resource_doc in completion_context.iter_dependency_and_init_resource_docs(
+        dependency_graph
+    ):
+        new_ctx = completion_context.create_copy(resource_doc)
+        _collect_current_doc_variables(new_ctx, collector)
+
+    for node, variable_doc in dependency_graph.iter_all_variable_imports_as_docs():
+        if variable_doc is None:
+            # Note that 'None' documents will only be given for the
+            # initial context (so, it's ok to use `completion_context`
+            # in this case).
+            from robot.api import Token
+
+            node_name_tok = node.get_token(Token.NAME)
+            if node_name_tok is not None:
+
+                (
+                    _value,
+                    token_errors,
+                ) = completion_context.token_value_and_unresolved_resolving_variables(
+                    node_name_tok
+                )
+
+                if token_errors:
+                    for token_error in token_errors:
+                        collector.on_unresolved_variable_import(
+                            completion_context,
+                            node.name,
+                            token_error.lineno,
+                            token_error.lineno,
+                            token_error.col_offset,
+                            token_error.end_col_offset,
+                            f"\nUnable to statically resolve variable: {token_error.value}.\nPlease set the `{token_error.value[2:-1]}` value in `robot.variables`.",
+                        )
+
+                else:
+                    collector.on_unresolved_variable_import(
+                        completion_context,
+                        node.name,
+                        node_name_tok.lineno,
+                        node_name_tok.lineno,
+                        node_name_tok.col_offset,
+                        node_name_tok.end_col_offset,
+                        None,
+                    )
+            else:
+                collector.on_unresolved_variable_import(
+                    completion_context,
+                    node.name,
+                    node.lineno,
+                    node.end_lineno,
+                    node.col_offset,
+                    node.end_col_offset,
+                    None,
+                )
+            continue
+        _collect_variables_from_variable_import_doc(variable_doc, collector)
+
+
+def _collect_global_variables_from_document_context(
     completion_context: ICompletionContext,
     collector: IVariablesCollector,
     only_current_doc=False,
@@ -293,65 +315,9 @@ def _collect_variables_from_document_context(
     _collect_current_doc_variables(completion_context, collector)
 
     if not only_current_doc:
-        dependency_graph = completion_context.collect_dependency_graph()
-
-        for resource_doc in completion_context.iter_dependency_and_init_resource_docs(
-            dependency_graph
-        ):
-            new_ctx = completion_context.create_copy(resource_doc)
-            _collect_current_doc_variables(new_ctx, collector)
-
-        for node, variable_doc in dependency_graph.iter_all_variable_imports_as_docs():
-            if variable_doc is None:
-                # Note that 'None' documents will only be given for the
-                # initial context (so, it's ok to use `completion_context`
-                # in this case).
-                from robot.api import Token
-
-                node_name_tok = node.get_token(Token.NAME)
-                if node_name_tok is not None:
-
-                    (
-                        _value,
-                        token_errors,
-                    ) = completion_context.token_value_and_unresolved_resolving_variables(
-                        node_name_tok
-                    )
-
-                    if token_errors:
-                        for token_error in token_errors:
-                            collector.on_unresolved_variable_import(
-                                completion_context,
-                                node.name,
-                                token_error.lineno,
-                                token_error.lineno,
-                                token_error.col_offset,
-                                token_error.end_col_offset,
-                                f"\nUnable to statically resolve variable: {token_error.value}.\nPlease set the `{token_error.value[2:-1]}` value in `robot.variables`.",
-                            )
-
-                    else:
-                        collector.on_unresolved_variable_import(
-                            completion_context,
-                            node.name,
-                            node_name_tok.lineno,
-                            node_name_tok.lineno,
-                            node_name_tok.col_offset,
-                            node_name_tok.end_col_offset,
-                            None,
-                        )
-                else:
-                    collector.on_unresolved_variable_import(
-                        completion_context,
-                        node.name,
-                        node.lineno,
-                        node.end_lineno,
-                        node.col_offset,
-                        node.end_col_offset,
-                        None,
-                    )
-                continue
-            _collect_variables_from_variable_import_doc(variable_doc, collector)
+        collect_global_variables_from_document_dependencies(
+            completion_context, collector
+        )
 
 
 def _collect_arguments(
@@ -361,7 +327,7 @@ def _collect_arguments(
 ):
     from robotframework_ls.impl import ast_utils
 
-    for arg_token, var_identifier in ast_utils.iter_keyword_arguments_as_tokens(
+    for arg_token, _var_identifier in ast_utils.iter_keyword_arguments_as_tokens(
         node, tokenize_keyword_name=True
     ):
         name = arg_token.value
@@ -376,29 +342,6 @@ def _collect_arguments(
             collector.on_variable(variable_found)
 
 
-def _collect_from_settings(
-    completion_context: ICompletionContext, collector: IVariablesCollector
-):
-    from robotframework_ls.impl.robot_lsp_constants import OPTION_ROBOT_VARIABLES
-
-    config = completion_context.config
-    if config is not None:
-        robot_variables = config.get_setting(OPTION_ROBOT_VARIABLES, dict, {})
-        for key, val in robot_variables.items():
-            if collector.accepts(key):
-                collector.on_variable(VariableFoundFromSettings(key, val))
-
-
-def _collect_from_builtins(
-    completion_context: ICompletionContext, collector: IVariablesCollector
-):
-    from robotframework_ls.impl.robot_constants import get_builtin_variables
-
-    for key, val in get_builtin_variables():
-        if collector.accepts(key):
-            collector.on_variable(VariableFoundFromBuiltins(key, val))
-
-
 def collect_variables(
     completion_context: ICompletionContext,
     collector: IVariablesCollector,
@@ -411,16 +354,26 @@ def collect_variables(
     collect_global_variables(completion_context, collector, only_current_doc)
 
 
-def _collect_from_arguments_files(
-    completion_context: ICompletionContext, collector: IVariablesCollector
-):
-    if not completion_context.variables_from_arguments_files_loader:
-        return
+def _collect_global_static_variables(completion_context, collector):
+    for (
+        var_found
+    ) in completion_context.get_settings_normalized_var_name_to_var_found().values():
+        if collector.accepts(var_found.variable_name):
+            collector.on_variable(var_found)
 
-    for c in completion_context.variables_from_arguments_files_loader:
-        for variable in c.get_variables():
-            if collector.accepts(variable.variable_name):
-                collector.on_variable(variable)
+    for var_found in completion_context.get_builtins_normalized_var_name_to_var_found(
+        False
+    ).values():
+        if collector.accepts(var_found.variable_name):
+            collector.on_variable(var_found)
+
+    for (
+        var_found
+    ) in (
+        completion_context.get_arguments_files_normalized_var_name_to_var_found().values()
+    ):
+        if collector.accepts(var_found.variable_name):
+            collector.on_variable(var_found)
 
 
 def collect_global_variables(
@@ -428,14 +381,12 @@ def collect_global_variables(
     collector: IVariablesCollector,
     only_current_doc=False,
 ):
-    _collect_variables_from_document_context(
+    _collect_global_variables_from_document_context(
         completion_context, collector, only_current_doc=only_current_doc
     )
 
     if not only_current_doc:
-        _collect_from_settings(completion_context, collector)
-        _collect_from_builtins(completion_context, collector)
-        _collect_from_arguments_files(completion_context, collector)
+        _collect_global_static_variables(completion_context, collector)
 
 
 def collect_local_variables(
@@ -465,6 +416,14 @@ def collect_local_variables(
 
     _collect_arguments(completion_context, stack_node, collector)
 
+    accept_sets_in = {
+        normalize_robot_name("Set Local Variable"),
+    }
+
+    _collect_variables_from_set_keywords(
+        stack_node, completion_context, collector, accept_sets_in
+    )
+
 
 def complete(completion_context: ICompletionContext) -> List[CompletionItemTypedDict]:
     from robotframework_ls.impl.string_matcher import RobotStringMatcher
